/*	AdditionalKey
	
	このファイルを読み込み、
	以下の2点を行うと、キーボードのみから、マウスとタッチ入力に対応となる。

	1. ソフトウェアキーボードのHTMLを配置する。
	
		<table id="softwarePad" width="100%" height="100">
			<tr>
				<td></td>
				<td id="softwareKey38">↑</td>
				<td></td>
			</tr>
			<tr>
				<td id="softwareKey37">←</td>
				<td id="softwareKey40">↓</td>
				<td id="softwareKey39">→</td>
			</tr>
		</table>
		
		※以下の関数で指定するので、例のようなidは必要。

	
	2. 以下のようなJavaScriptを記述する。
	
		ak = new AdditionalKey();
		ak.setSoftwareKey( "softwareKey37", onkeydownx, onkeyupx, 37 );
		ak.setSoftwareKey( "softwareKey38", onkeydownx, onkeyupx, 38 );
		ak.setSoftwareKey( "softwareKey39", onkeydownx, onkeyupx, 39 );
		ak.setSoftwareKey( "softwareKey40", onkeydownx, onkeyupx, 40 );
		ak.tweakSoftwarePad( "softwarePad" );
		
		※"softwareKey37"等は、たとえば上記HTMLのID
		※onkeydownx, onkeyupx は、キーイベントハンドラ関数。
		　window.onkeydown を指定してもよい。
		※例の 37, 38, 39, 40 は↑↓←→キーのキー番号
		※tweakSoftwarePadはなくてもよいが、あればテキスト選択防止、スライド押しができるようになる。
	
	
	特徴：
	上記の例のスクリプトを貼り付けても貼り付けなくても、どちらにせよ
	元のスクリプトはエラーを出さずに動作できる。
	つまり依存度が非常に低いので、プログラマーの負担にならない。
*/


	//雑用関数

	function getPseudoStyle( keySelector ) {
	/*
		AdditionalKey.prototype.setSoftwareKey のコメント
		//スマートホンの疑似要素が上手く動かなかったので特別対応
		にて2回使用。それ以外では使用していない。
		
		usage: let styleHover = getPseudoStyle( "td:hover" );
		usage: let styleDefault = getPseudoStyle( "td" );

		getComputedStyle( element, ":hover" ) として、
		その要素の疑似要素の内容が取得できないとき、この関数を使えば取得できる。
		なお、ページ上で同じセレクタで重複して設定されている場合は後のほうが有効となる。
		
		※getComputedStyleは、疑似要素のbeforeなどでcontentなど取得できるかもしれないが、
		hoverはbackground-colorが取得できず、疑似要素ではないほうのbackground-colorが取得されてしまった。
	*/

		let res = new Object();
		/*	ページ上のスタイルタグをすべて検索する。
			keySelectorを見つけたら、そのスタイルの各名前と値の組をresに入れる。
		*/
		for( let i = 0; i < document.styleSheets.length; i++ ){
			let ss = document.styleSheets[ i ];
			for( let j = 0; j < ss.cssRules.length; j++ ){    
				let selectorText = ss.cssRules[ j ].selectorText;
				let selectors = selectorText.split( /,/ );
				selectors = selectors.map( selector => selector.replace( /^ +| +$/g, "" ) );
				if( selectors.indexOf( keySelector ) > -1 ){
					let text = ss.cssRules[ j ].style.cssText;
					let pairs = text.split( /;/ );
					for( let k = 0; k < pairs.length; k++ ) {
						let pair = pairs[ k ].split( /:/ );
						let name = pair[ 0 ].replace( /^ /, "" );
						let value = pair[ 1 ];
						res[ name ] = value;
					}//for k
				}//if
			}//for j
		}//for i
		return res;
	}



AdditionalKey = function() {
	this.tableFlg = 0;	//
	
	//タッチ対応
	this.touchFunction1 = function( te ) {
		for( var i = 0; i < te.changedTouches.length; i++ ) {
			var touch = te.changedTouches[ i ];
			touch.target.onmousedown( te );
		}
		return false;
	}
	this.touchFunction2 = function( te ) {
		for( var i = 0; i < te.changedTouches.length; i++ ) {
			var touch = te.changedTouches[ i ];
			touch.target.onmouseup( te );
		}
	}
	addEventListener( "touchstart", this.touchFunction1 );
	addEventListener( "touchend", this.touchFunction2 );
}


AdditionalKey.prototype.setSoftwareKey = function( elementId, onkeydownx, onkeyupx, keyNum ) {
	let element = document.getElementById( elementId );
	let AK = this;
	
	//スマートホンの疑似要素が上手く動かなかったので特別対応
		
		//疑似要素で変更したいCSSプロパティ
		let props = [ "background-color", "font-size", "font-weight" ];
		
		//td:hoverのCSSプロパティを取得
		let styleHover = getPseudoStyle( "td:hover" );
		let styleOnmousedown = new Object();
		props.map( name => styleOnmousedown[ name ] = styleHover[ name ] );

		//tdのCSSプロパティを取得
		let styleDefault = getPseudoStyle( "td" );
		let styleOnmouseup = new Object();
		props.map( name => styleOnmouseup[ name ] = styleDefault[ name ] );
		
		//疑似要素の動きの代替
		element.addEventListener( "touchstart", function( e ) {
			props.map( name => e.target.style[ name ] = styleOnmousedown[ name ] );
		} );
		element.addEventListener( "touchend", function( e ) {
			props.map( name => e.target.style[ name ] = styleOnmouseup[ name ] );
		} );
	
	
	//こちらはソフトウェアキーの実装
	
	element.onmousedown = function( e ) {
		e.preventDefault();
		onkeydownx( { which:keyNum } ) };
	element.onmouseover = function( e ) {
		if( ! AK.tableFlg ) return;
		onkeydownx( { which:keyNum } );
	}
	element.onmouseup = function( e ) { onkeyupx( { which:keyNum } ) }
	element.onmouseout = function( e ) { onkeyupx( { which:keyNum } ) }
}



AdditionalKey.prototype.tweakSoftwarePad = function( elementId ) {

	let element = document.getElementById( elementId );
	let AK = this;
	
	//マウスを押したままスライドするとスライド先のボタンを押す仕組み（タッチでは不可）
	element.onmousedown = function( e ) {  AK.tableFlg = 1 }
	element.onmouseup = function( e ) { AK.tableFlg = 0 }
	
	//テキスト選択を防止
	element.style.WebkitUserSelect = "none";
	element.style.MozUserSelect = "none";
	element.style.MsUserSelect = "none";
	element.style.userSelect = "none";
}

